CraftBarData = {};

-- Bindings
BINDING_HEADER_CRAFTBARHEADER = "CraftBar";
BINDING_NAME_CRAFTBARSNAP = "MouseSnap";

local MASQUE = LibStub("Masque", true);
local buttongroup = nil;

local initialized = false;

local buttonIndex = 1;
local craftbar_professions = {
    ['101Pick Lock'] = {spellid=1804},
    ['102Runeforging'] = {spellid=53428, recipes=true},
}

local craftbar_buttons = {};

function foo(str)
    local id = tonumber(str);
    local name =  GetSpellInfo(id);
    DEFAULT_CHAT_FRAME:AddMessage("spell id "..str..": "..(name or "nil"), 1.0, 1.0, 1.0, 1, 10);
end

function craftbar_spellid(spellName)
    local spellLink = GetSpellLink(spellName);
    if spellLink then
        return tonumber(spellLink:match("spell:(%d+)"));
    end
    
    return 0;
end

local debug = false;
function craftbar_debug(value, name)
    if (not debug) then
        return;
    end

    if (name) then
        DEFAULT_CHAT_FRAME:AddMessage("debug "..name..": "..(value or "nil"), 1.0, 1.0, 1.0, 1, 10);
    else
        DEFAULT_CHAT_FRAME:AddMessage("debug: "..(value or "nil"), 1.0, 1.0, 1.0, 1, 10);
    end
end

-- functions
function CraftBar_OnLoad(self)
    self:RegisterEvent("ADDON_LOADED");

    self:RegisterEvent("PLAYER_ENTER_COMBAT");
    self:RegisterEvent("PLAYER_LEAVE_COMBAT");
    self:RegisterEvent("PLAYER_REGEN_DISABLED");
    self:RegisterEvent("PLAYER_REGEN_ENABLED");

    self:RegisterEvent("TRADE_SKILL_SHOW");

    --self:RegisterAllEvents();

    SLASH_CRAFTBAR1 = "/craftbar";
    SLASH_CRAFTBAR2 = "/craft";
    SlashCmdList["CRAFTBAR"] = function(msg)
        CraftBar_SlashCommandHandler(msg);
    end
end

function CraftBar_OnEvent(self, event, ...)

    --craftbar_debug(event, "event");

    if (event == "ADDON_LOADED") then

        local addonName = ...;
        if (addonName == "CraftBar") then
            if (MASQUE) then
                buttongroup = MASQUE:Group("CraftBar");
            end
        end
    end

    if (event == "PLAYER_ENTER_COMBAT" or event == "PLAYER_REGEN_DISABLED") then

        if (InCombatLockdown() == nil) then
            local frame = getglobal("CraftBarFrame");
            frame:Hide();
        end
    end
end

function CraftBar_SlashCommandHandler(msg)
    if (msg == "reload") then
        ReloadUI();
    end
    if (msg == "togglerank") then
        if (CraftBarData.ShowRank) then
            CraftBarData.ShowRank = false
        else
            CraftBarData.ShowRank = true
        end
    end
end

function CraftBar_AppendProfessionInfo(index)
    if (index) then
        local name, texture, rank, maxRank, numSpells, spelloffset, skillLine, rankModifier, specializationIndex, specializationOffset = GetProfessionInfo(index);
    
        local spellid = craftbar_spellid(name);
        
        local index1 = spelloffset + 1;
        local index2 = spelloffset + 2;
        
        local _, id = GetSpellBookItemInfo(index1, BOOKTYPE_PROFESSION);
        
        if (id) then
            craftbar_professions[CraftBar_CreateKeyForName(name)] = {spellid=id, rank=rank, maxRank=maxRank};
        end        
                
        if (numSpells == 2) then
            local _, id2 = GetSpellBookItemInfo(index2, BOOKTYPE_PROFESSION);
                    
            if (id2 and not IsPassiveSpell(index2, BOOKTYPE_PROFESSION)) then
                name = GetSpellBookItemName(index2, BOOKTYPE_PROFESSION);
                craftbar_professions[CraftBar_CreateKeyForName(name)] = {spellid=id2};
            end
        end
    end
end

function CraftBar_CreateKeyForName(name)
    local key = ""
    
    if (buttonIndex < 10) then
        key = "0"
    end
    
    local key = key..buttonIndex.."_"..name;
    
    buttonIndex = buttonIndex + 1;
    
    return key;
end

function CraftBar_initButtons()
    CraftBarFrame:Hide();
    
    -- init professions
    local prof1, prof2, arch, fish, cook, firstAid = GetProfessions();
    
    CraftBar_AppendProfessionInfo(prof1);
    CraftBar_AppendProfessionInfo(prof2);
    CraftBar_AppendProfessionInfo(arch);
    CraftBar_AppendProfessionInfo(fish);
    CraftBar_AppendProfessionInfo(cook);
    CraftBar_AppendProfessionInfo(firstAid);

    for name, data in craftbar_pairsByKeys(craftbar_professions) do
        local spellid = data.spellid;
        local rank = data.rank;
        
        local spellname, _, texture = GetSpellInfo(spellid);

        -- Button in der Buttonleiste
        local button1 = CreateFrame("Button", "CraftBar"..name, CraftBarFrame, "CraftBarButtonTemplate");

        button1:RegisterForClicks("AnyUp");
        button1.rangeTimer = -1;

        button1:RegisterEvent("PLAYER_TARGET_CHANGED");
        button1:RegisterEvent("ACTIONBAR_UPDATE_USABLE");
        button1:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");

        button1:SetAttribute("type", "spell");
        button1:SetAttribute("spell", spellname);
        button1:SetAttribute("id", spellid);
        
        if (rank and CraftBarData.ShowRank) then
            button1.Count:SetText(""..rank);
        end

        if (MASQUE) then 
            buttongroup:AddButton(button1);
        end

        _G[button1:GetName().."Icon"]:SetTexture(texture);

        craftbar_buttons[name] = {bar=button1};
    end

    initialized = true;
end

function craftbar_pairsByKeys(t, f)
    local a = {};

    for n in pairs(t) do
        table.insert(a, n)
    end

    table.sort(a, f);
    local i = 0;      -- iterator variable
    local iter = function ()   -- iterator function
        i = i + 1;
        if a[i] == nil then
            return nil;
        else
            return a[i], t[a[i]];
        end
    end

    return iter;
end

function CraftBar_initCraftFrame()
    if (not initialized) then
        CraftBar_initButtons();
    end

    local count = 0;

    for name, data in craftbar_pairsByKeys(craftbar_professions) do
        local spellid = data.spellid;
        local spellname, rank, texture, cost, isFunnel, powerType, castTime, minRange, maxRange = GetSpellInfo(spellid);
        local usable, nomana = IsUsableSpell(spellname);

        local button = craftbar_buttons[name].bar;

        if (usable) then
            button:ClearAllPoints();
            button:SetPoint("LEFT", (6 + 38 * count), 0);

            _G[button:GetName().."Icon"]:SetTexture(GetSpellTexture(spellname));

            button:Show();
            count = count + 1;
        else
            button:Hide();
        end
    end

    CraftBarFrame:SetWidth(10 + 38 * count);

    return count;
end

function CraftBar_SnapDown()

    if (InCombatLockdown()) then
        UIErrorsFrame:AddMessage(ERR_NOT_IN_COMBAT, 1.0, 0.0, 0.0, 1, 10);
    end

    local frame = getglobal("CraftBarFrame");

    local count = CraftBar_initCraftFrame();

    if (count < 1) then
        frame:Hide();
        return;
    end

    def_posx = frame:GetLeft();
    def_posy = frame:GetBottom();

    local x, y = GetCursorPosition();
    local scale = UIParent:GetEffectiveScale();

    x = x / scale;
    y = y / scale;

    frame:ClearAllPoints();
    frame:SetPoint("CENTER", UIParent, "BOTTOMLEFT", x, y);

    frame:Show();
end

function CraftBar_SnapUp()
    local frame = getglobal("CraftBarFrame");
    frame:ClearAllPoints();
    frame:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", def_posx, def_posy);

    if (not InCombatLockdown()) then
        frame:Hide();
    end
end

function CraftBar_MouseOver(button)
    CraftBar_SetTooltip(button);
end

function CraftBar_MouseOut(button)
    GameTooltip:FadeOut();
    selectedButton = nil;
end

function CraftBar_SetTooltip(button)

    local spellid = button:GetAttribute("id");

    GameTooltip_SetDefaultAnchor(GameTooltip, button);

    GameTooltip:SetSpellByID(spellid);

    GameTooltip:AppendText("");
end
